/* simple event library.
 * Written by Michael Tokarev <mjt@corpit.ru>
 * LGPL
 */

#ifndef _EVENT_H_INCLUDED
#define _EVENT_H_INCLUDED

#include <sys/types.h>
struct timeval;

typedef enum {
  EVENT_TIMER	= 0,
  EVENT_READ	= 1,
  EVENT_WRITE	= 2,
  EVENT_RDWR	= EVENT_READ|EVENT_WRITE,
  EVENT_EXCPT	= 4,  /* cannot be used in event_io_register() */
  EVENT_ERROR	= 8   /* error: errno is set appropriately */
} event_t;

typedef struct _event_ctx event_ctx;

typedef void event_callback(event_ctx *ectx, int fd, event_t event, void *ctx);

event_ctx *event_ctx_new();
void event_ctx_free(event_ctx *ectx);

time_t event_time(const event_ctx *ectx);
const struct timeval *event_timeval(const event_ctx *ectx);

int event_events(const event_ctx *ectx);

int event_wait(event_ctx *ectx, int timeout);

int event_request_timer(event_ctx *ectx, int timeout,
                        event_callback *cbck, void *ctx,
                        time_t now);
int event_request_timer_tv(event_ctx *ectx, int timeout,
                           event_callback *cbck, void *ctx,
                           const struct timeval *tvnow);

int event_request_io(event_ctx *ectx, int fd, event_t event,
                     event_callback *cbck, void *ctx);

event_callback *event_io_cabck(const event_ctx *ectx, int fd);
void *event_io_ctx(const event_ctx *ectx, int fd);
int event_io_wakeup(event_ctx *ectx, int fd, event_t event);

#endif
